@@ -1,18 +1,8 @@
 //
 // Copyright 2013-2015 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "e300_impl.hpp"
@@ -25,7 +15,7 @@
 #include "e300_common.hpp"
 #include "e300_remote_codec_ctrl.hpp"
 
-#include <uhd/utils/msg.hpp>
+
 #include <uhd/utils/log.hpp>
 #include <uhd/utils/static.hpp>
 #include <uhd/utils/paths.hpp>
@@ -42,9 +32,10 @@
 #include <boost/bind.hpp>
 #include <boost/make_shared.hpp>
 #include <boost/assign/list_of.hpp>
-#include <boost/thread/thread.hpp> //sleep
 #include <boost/asio.hpp>
 #include <fstream>
+#include <chrono>
+#include <thread>
 
 using namespace uhd;
 using namespace uhd::usrp;
@@ -75,11 +66,11 @@ static std::vector<std::string> discover_ip_addrs(
     try {
         udp_bcast_xport = uhd::transport::udp_simple::make_broadcast(addr_hint, port);
     } catch(const std::exception &e) {
-        UHD_MSG(error) << boost::format("Cannot open UDP transport on %s for discovery\n%s")
-        % addr_hint % e.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << boost::format("Cannot open UDP transport on %s for discovery%s")
+        % addr_hint % e.what() ;
         return addrs;
     } catch(...) {
-        UHD_MSG(error) << "E300 Network discovery unknown error" << std::endl;
+        UHD_LOGGER_ERROR("E300") << "E300 Network discovery unknown error";
         return addrs;
     }
 
@@ -95,10 +86,10 @@ static std::vector<std::string> discover_ip_addrs(
     try {
     udp_bcast_xport->send(boost::asio::buffer(&req, sizeof(req)));
     } catch (const std::exception &ex) {
-        UHD_MSG(error) << "E300 Network discovery error " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "E300 Network discovery error " << ex.what();
         return addrs;
     } catch(...) {
-        UHD_MSG(error) << "E300 Network discovery unknown error" << std::endl;
+        UHD_LOGGER_ERROR("E300") << "E300 Network discovery unknown error";
         return addrs;
     }
 
@@ -130,7 +121,7 @@ device_addrs_t e300_find(const device_addr_t &multi_dev_hint)
     if (hints.size() > 1) {
         device_addrs_t found_devices;
         std::string err_msg;
-        BOOST_FOREACH(const device_addr_t &hint_i, hints)
+        for(const device_addr_t &hint_i:  hints)
         {
             device_addrs_t found_devices_i = e300_find(hint_i);
             if(found_devices_i.size() != 1)
@@ -165,7 +156,7 @@ device_addrs_t e300_find(const device_addr_t &multi_dev_hint)
     if (not loopback_only) {
         // if no address or node has been specified, send a broadcast
         if ((not hint.has_key("addr")) and (not hint.has_key("node"))) {
-            BOOST_FOREACH(const if_addrs_t &if_addrs, get_if_addrs())
+            for(const if_addrs_t &if_addrs:  get_if_addrs())
             {
                 // avoid the loopback device
                 if (is_loopback(if_addrs))
@@ -187,7 +178,7 @@ device_addrs_t e300_find(const device_addr_t &multi_dev_hint)
         std::vector<std::string> ip_addrs = discover_ip_addrs(
             hint["addr"], E300_SERVER_I2C_PORT);
 
-        BOOST_FOREACH(const std::string &ip_addr, ip_addrs)
+        for(const std::string &ip_addr:  ip_addrs)
         {
             device_addr_t new_addr;
             new_addr["type"] = "e3x0";
@@ -200,7 +191,7 @@ device_addrs_t e300_find(const device_addr_t &multi_dev_hint)
                 const mboard_eeprom_t eeprom = eeprom_manager.get_mb_eeprom();
                 new_addr["name"] = eeprom["name"];
                 new_addr["serial"] = eeprom["serial"];
-                new_addr["product"] = eeprom["product"];
+                new_addr["product"] = eeprom_manager.get_mb_type_string();
             } catch (...) {
                 // set these values as empty string, so the device may still be found
                 // and the filters below can still operate on the discovered device
@@ -236,7 +227,7 @@ device_addrs_t e300_find(const device_addr_t &multi_dev_hint)
             const mboard_eeprom_t eeprom = eeprom_manager.get_mb_eeprom();
             new_addr["name"] = eeprom["name"];
             new_addr["serial"] = eeprom["serial"];
-            new_addr["product"] = eeprom["product"];
+            new_addr["product"] = eeprom_manager.get_mb_type_string();
         } catch (...) {
             // set these values as empty string, so the device may still be found
             // and the filters below can still operate on the discovered device
@@ -260,7 +251,7 @@ device_addrs_t e300_find(const device_addr_t &multi_dev_hint)
  **********************************************************************/
 static device::sptr e300_make(const device_addr_t &device_addr)
 {
-    UHD_LOG << "e300_make with args " << device_addr.to_pp_string() << std::endl;
+    UHD_LOGGER_DEBUG("E300")<< "e300_make with args " << device_addr.to_pp_string() ;
     if(device_addr.has_key("server"))
         throw uhd::runtime_error(
             str(boost::format("Please run the server executable \"%s\"")
@@ -295,8 +286,8 @@ void get_e3x0_fpga_images(const uhd::device_addr_t &device_addr,
         break;
     case e300_eeprom_manager::UNKNOWN:
     default:
-        UHD_MSG(warning) << "Unknown motherboard type, loading e300 image."
-                             << std::endl;
+        UHD_LOGGER_WARNING("E300") << "Unknown motherboard type, loading e300 image."
+                             ;
         fpga_image = device_addr.cast<std::string>("fpga",
             find_image_path(E300_FPGA_FILE_NAME));
         idle_image = find_image_path(E3XX_SG1_FPGA_IDLE_FILE_NAME);
@@ -323,7 +314,14 @@ e300_impl::e300_impl(const uhd::device_addr_t &device_addr)
         _do_not_reload = device_addr.has_key("no_reload_fpga");
         if (not _do_not_reload) {
             std::string fpga_image;
-            get_e3x0_fpga_images(device_addr,
+
+            // need to re-read product ID code because of conversion into string in find function
+            e300_eeprom_manager eeprom_manager(i2c::make_i2cdev(E300_I2CDEV_DEVICE));
+            const mboard_eeprom_t eeprom = eeprom_manager.get_mb_eeprom();
+            device_addr_t device_addr_cp;
+            device_addr_cp["product"] = eeprom["product"];
+
+            get_e3x0_fpga_images(device_addr_cp,
                                  fpga_image,
                                  _idle_image);
             common::load_fpga_image(fpga_image);
@@ -398,27 +396,27 @@ e300_impl::e300_impl(const uhd::device_addr_t &device_addr)
         ad9361_params::sptr client_settings = boost::make_shared<e300_ad9361_client_t>();
         _codec_ctrl = ad9361_ctrl::make_spi(client_settings, spi::make(E300_SPIDEV_DEVICE), 1);
         // This is horrible ... why do I have to sleep here?
-        boost::this_thread::sleep(boost::posix_time::milliseconds(100));
+        std::this_thread::sleep_for(std::chrono::milliseconds(100));
         _eeprom_manager = boost::make_shared<e300_eeprom_manager>(i2c::make_i2cdev(E300_I2CDEV_DEVICE));
         _sensor_manager = e300_sensor_manager::make_local(_global_regs);
     }
     _codec_mgr = ad936x_manager::make(_codec_ctrl, fpga::NUM_RADIOS);
 
 #ifdef E300_GPSD
-    UHD_MSG(status) << "Detecting internal GPS " << std::flush;
+    UHD_LOGGER_INFO("E300") << "Detecting internal GPS ";
     try {
         if (_xport_path == AXI)
             _gps = gpsd_iface::make("localhost", 2947);
         else
             _gps = gpsd_iface::make(device_addr["addr"], 2947);
     } catch (std::exception &e) {
-        UHD_MSG(error) << "An error occured making GPSDd interface: " << e.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "An error occured making GPSDd interface: " << e.what();
     }
 
     if (_gps) {
         for (size_t i = 0; i < _GPS_TIMEOUT; i++)
         {
-            boost::this_thread::sleep(boost::posix_time::seconds(1));
+            std::this_thread::sleep_for(std::chrono::seconds(1));
             if (!_gps->gps_detected())
                 std::cout << "." << std::flush;
             else {
@@ -426,12 +424,12 @@ e300_impl::e300_impl(const uhd::device_addr_t &device_addr)
                 break;
             }
         }
-        UHD_MSG(status) << (_gps->gps_detected() ? "found" : "not found") << std::endl;
+        UHD_LOGGER_INFO("E300") << "GPSDO " << (_gps->gps_detected() ? "found" : "not found");
     }
 #endif
 
     // Verify we can talk to the e300 core control registers ...
-    UHD_MSG(status) << "Initializing core control..." << std::endl;
+    UHD_LOGGER_INFO("E300") << "Initializing core control...";
     this->_register_loopback_self_test(_global_regs);
 
     // Verify fpga compatibility version matches at least for the major
@@ -468,14 +466,14 @@ e300_impl::e300_impl(const uhd::device_addr_t &device_addr)
     // and do the misc mboard sensors
     ////////////////////////////////////////////////////////////////////
     _tree->create<int>(mb_path / "sensors");
-    BOOST_FOREACH(const std::string &name, _sensor_manager->get_sensors())
+    for(const std::string &name:  _sensor_manager->get_sensors())
     {
         _tree->create<sensor_value_t>(mb_path / "sensors" / name)
             .set_publisher(boost::bind(&e300_sensor_manager::get_sensor, _sensor_manager, name));
     }
 #ifdef E300_GPSD
     if (_gps) {
-        BOOST_FOREACH(const std::string &name, _gps->get_sensors())
+        for(const std::string &name:  _gps->get_sensors())
         {
             _tree->create<sensor_value_t>(mb_path / "sensors" / name)
                 .set_publisher(boost::bind(&gpsd_iface::get_sensor, _gps, name));
@@ -511,27 +509,55 @@ e300_impl::e300_impl(const uhd::device_addr_t &device_addr)
         this->_setup_radio(instance);
 
     //now test each radio module's connection to the codec interface
-    BOOST_FOREACH(radio_perifs_t &perif, _radio_perifs)
-    {
+    for (radio_perifs_t &perif : _radio_perifs) {
         _codec_mgr->loopback_self_test(
-            boost::bind(
-                &radio_ctrl_core_3000::poke32, perif.ctrl, radio::sr_addr(radio::CODEC_IDLE), _1
-            ),
-            boost::bind(&radio_ctrl_core_3000::peek64, perif.ctrl, radio::RB64_CODEC_READBACK)
+            [&perif](const uint32_t value){
+                perif.ctrl->poke32(radio::sr_addr(radio::CODEC_IDLE), value);
+            },
+            [&perif](){
+                return perif.ctrl->peek64(radio::RB64_CODEC_READBACK);
+            }
         );
     }
     ////////////////////////////////////////////////////////////////////
     // internal gpios
     ////////////////////////////////////////////////////////////////////
     gpio_atr_3000::sptr fp_gpio = gpio_atr_3000::make(_radio_perifs[0].ctrl, radio::sr_addr(radio::FP_GPIO), radio::RB32_FP_GPIO);
-    BOOST_FOREACH(const gpio_attr_map_t::value_type attr, gpio_attr_map)
-    {
-        _tree->create<uint32_t>(mb_path / "gpio" / "INT0" / attr.second)
-            .add_coerced_subscriber(boost::bind(&gpio_atr_3000::set_gpio_attr, fp_gpio, attr.first, _1))
-            .set(0);
+    for(const auto& attr:  gpio_attr_map){
+        switch (attr.first){
+                case usrp::gpio_atr::GPIO_SRC:
+                    _tree->create<std::vector<std::string>>(mb_path / "gpio" / "INT0" / attr.second)
+                         .set(std::vector<std::string>(32, usrp::gpio_atr::default_attr_value_map.at(attr.first)))
+                         .add_coerced_subscriber([this](const std::vector<std::string>&){
+                            throw uhd::runtime_error("This device does not support setting the GPIO_SRC attribute.");
+                         });
+                    break;
+                case usrp::gpio_atr::GPIO_CTRL:
+                case usrp::gpio_atr::GPIO_DDR:
+                    _tree->create<std::vector<std::string>>(mb_path / "gpio" / "INT0" / attr.second)
+                         .set(std::vector<std::string>(32, usrp::gpio_atr::default_attr_value_map.at(attr.first)))
+                         .add_coerced_subscriber([this, fp_gpio, attr](const std::vector<std::string> str_val){
+                            uint32_t val = 0;
+                            for(size_t i = 0 ; i < str_val.size() ; i++){
+                                val += usrp::gpio_atr::gpio_attr_value_pair.at(attr.second).at(str_val[i])<<i;
+                            }
+                            fp_gpio->set_gpio_attr(attr.first, val);
+                         });
+                    break;
+                case usrp::gpio_atr::GPIO_READBACK:
+                    _tree->create<uint8_t>(mb_path / "gpio" / "INT0" / "READBACK")
+                        .set_publisher([this, fp_gpio](){
+                            return fp_gpio->read_gpio();
+                         });
+                    break;
+                default:
+                    _tree->create<uint32_t>(mb_path / "gpio" / "INT0" / attr.second)
+                         .set(0)
+                         .add_coerced_subscriber([this, fp_gpio, attr](const uint32_t val){
+                             fp_gpio->set_gpio_attr(attr.first, val);
+                         });
+            }
     }
-    _tree->create<uint8_t>(mb_path / "gpio" / "INT0" / "READBACK")
-        .set_publisher(boost::bind(&gpio_atr_3000::read_gpio, fp_gpio));
 
 
     ////////////////////////////////////////////////////////////////////
@@ -625,11 +651,11 @@ e300_impl::e300_impl(const uhd::device_addr_t &device_addr)
 
     // subdev spec contains full width of selections
     subdev_spec_t rx_spec, tx_spec;
-    BOOST_FOREACH(const std::string &fe, _tree->list(mb_path / "dboards" / "A" / "rx_frontends"))
+    for(const std::string &fe:  _tree->list(mb_path / "dboards" / "A" / "rx_frontends"))
     {
         rx_spec.push_back(subdev_spec_pair_t("A", fe));
     }
-    BOOST_FOREACH(const std::string &fe, _tree->list(mb_path / "dboards" / "A" / "tx_frontends"))
+    for(const std::string &fe:  _tree->list(mb_path / "dboards" / "A" / "tx_frontends"))
     {
         tx_spec.push_back(subdev_spec_pair_t("A", fe));
     }
@@ -692,11 +718,11 @@ void e300_impl::_enforce_tick_rate_limits(
 
 double e300_impl::_set_tick_rate(const double rate)
 {
-    UHD_MSG(status) << "Asking for clock rate " << rate/1e6 << " MHz\n";
+    UHD_LOGGER_INFO("E300") << "Asking for clock rate " << rate/1e6 << " MHz\n";
     _tick_rate = _codec_ctrl->set_clock_rate(rate);
-    UHD_MSG(status) << "Actually got clock rate " << _tick_rate/1e6 << " MHz\n";
+    UHD_LOGGER_INFO("E300") << "Actually got clock rate " << _tick_rate/1e6 << " MHz\n";
 
-    BOOST_FOREACH(radio_perifs_t &perif, _radio_perifs)
+    for(radio_perifs_t &perif:  _radio_perifs)
     {
         perif.time64->set_tick_rate(_tick_rate);
         perif.time64->self_test();
@@ -707,7 +733,7 @@ double e300_impl::_set_tick_rate(const double rate)
 void e300_impl::_register_loopback_self_test(wb_iface::sptr iface)
 {
     bool test_fail = false;
-    UHD_MSG(status) << "Performing register loopback test... " << std::flush;
+    UHD_LOGGER_INFO("E300") << "Performing register loopback test... ";
     size_t hash = size_t(time(NULL));
     for (size_t i = 0; i < 100; i++)
     {
@@ -716,7 +742,7 @@ void e300_impl::_register_loopback_self_test(wb_iface::sptr iface)
         test_fail = iface->peek32(radio::RB32_TEST) != uint32_t(hash);
         if (test_fail) break; //exit loop on any failure
     }
-    UHD_MSG(status) << ((test_fail)? " fail" : "pass") << std::endl;
+    UHD_LOGGER_INFO("E300") << "Register loopback test " << ((test_fail)? " failed" : "passed");
 }
 
 uint32_t e300_impl::_get_version(compat_t which)
@@ -740,26 +766,27 @@ std::string e300_impl::_get_version_hash(void)
         = _global_regs->peek32(global_regs::RB32_CORE_GITHASH);
     return str(boost::format("%7x%s")
         % (git_hash & 0x0FFFFFFF)
-        % ((git_hash & 0xF000000) ? "-dirty" : ""));
+        % ((git_hash & 0xF0000000) ? "-dirty" : ""));
 }
 
 uint32_t e300_impl::_allocate_sid(const sid_config_t &config)
 {
     const uint32_t stream = (config.dst_prefix | (config.router_dst_there << 2)) & 0xff;
 
+    const size_t sid_framer = _sid_framer++; //increment for next setup
     const uint32_t sid = 0
         | (E300_DEVICE_HERE << 24)
-        | (_sid_framer << 16)
+        | (sid_framer << 16)
         | (config.router_addr_there << 8)
         | (stream << 0)
     ;
-    UHD_LOG << std::hex
+    UHD_LOGGER_DEBUG("E300")<< std::hex
         << " sid 0x" << sid
-        << " framer 0x" << _sid_framer
+        << " framer 0x" << sid_framer
         << " stream 0x" << stream
         << " router_dst_there 0x" << int(config.router_dst_there)
         << " router_addr_there 0x" << int(config.router_addr_there)
-        << std::dec << std::endl;
+        << std::dec ;
 
     // Program the E300 to recognize it's own local address.
     _global_regs->poke32(global_regs::SR_CORE_XB_LOCAL, config.router_addr_there);
@@ -775,26 +802,23 @@ uint32_t e300_impl::_allocate_sid(const sid_config_t &config)
     _global_regs->poke32(XB_ADDR(E300_DEVICE_HERE),
                          config.router_dst_here);
 
-    UHD_LOG << std::hex
+    UHD_LOGGER_TRACE("E300") << std::hex
         << "done router config for sid 0x" << sid
-        << std::dec << std::endl;
-
-    //increment for next setup
-    _sid_framer++;
+        << std::dec ;
 
     return sid;
 }
 
 void e300_impl::_setup_dest_mapping(const uint32_t sid, const size_t which_stream)
 {
-    UHD_LOG << boost::format("Setting up dest map for 0x%lx to be stream %d")
-                                     % (sid & 0xff) % which_stream << std::endl;
+    UHD_LOGGER_DEBUG("E300") << boost::format("Setting up dest map for 0x%lx to be stream %d")
+                                     % (sid & 0xff) % which_stream ;
     _global_regs->poke32(DST_ADDR(sid & 0xff), which_stream);
 }
 
 void e300_impl::_update_time_source(const std::string &source)
 {
-    UHD_MSG(status) << boost::format("Setting time source to %s") % source << std::endl;
+    UHD_LOGGER_INFO("E300") << boost::format("Setting time source to %s") % source;
     if (source == "none" or source == "internal") {
         _misc.pps_sel = global_regs::PPS_INT;
 #ifdef E300_GPSD
@@ -811,7 +835,7 @@ void e300_impl::_update_time_source(const std::string &source)
 
 void e300_impl::_set_time(const uhd::time_spec_t& t)
 {
-    BOOST_FOREACH(radio_perifs_t &perif, _radio_perifs)
+    for(radio_perifs_t &perif:  _radio_perifs)
         perif.time64->set_time_sync(t);
     _misc.time_sync = 1;
     _update_gpio_state();
@@ -1037,7 +1061,7 @@ void e300_impl::_setup_radio(const size_t dspno)
     // create RF frontend interfacing
     ////////////////////////////////////////////////////////////////////
     static const std::vector<direction_t> dirs = boost::assign::list_of(RX_DIRECTION)(TX_DIRECTION);
-    BOOST_FOREACH(direction_t dir, dirs) {
+    for(direction_t dir:  dirs) {
         const std::string x = (dir == RX_DIRECTION) ? "rx" : "tx";
         const std::string key = boost::to_upper_copy(x) + std::string(((dspno == FE0)? "1" : "2"));
         const fs_path rf_fe_path
@@ -1118,7 +1142,7 @@ void e300_impl::_reset_codec_mmcm(void)
 {
     _misc.codec_arst = 1;
     _update_gpio_state();
-    boost::this_thread::sleep(boost::posix_time::milliseconds(10));
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
     _misc.codec_arst = 0;
     _update_gpio_state();
 }
