@@ -1,35 +1,25 @@
 //
 // Copyright 2015-2017 Ettus Research, A National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "x300_radio_ctrl_impl.hpp"
-
 #include "x300_dboard_iface.hpp"
-#include "wb_iface_adapter.hpp"
-#include "gpio_atr_3000.hpp"
-#include "apply_corrections.hpp"
 #include <uhd/usrp/dboard_eeprom.hpp>
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <uhd/usrp/dboard_iface.hpp>
 #include <uhd/rfnoc/node_ctrl_base.hpp>
 #include <uhd/transport/chdr.hpp>
 #include <uhd/utils/math.hpp>
+#include <uhdlib/rfnoc/wb_iface_adapter.hpp>
+#include <uhdlib/usrp/cores/gpio_atr_3000.hpp>
+#include <uhdlib/usrp/common/apply_corrections.hpp>
 #include <boost/algorithm/string.hpp>
 #include <boost/make_shared.hpp>
 #include <boost/date_time/posix_time/posix_time_io.hpp>
+#include <chrono>
+#include <thread>
 
 using namespace uhd;
 using namespace uhd::usrp;
@@ -44,7 +34,7 @@ static const size_t IO_MASTER_RADIO = 0;
 UHD_RFNOC_RADIO_BLOCK_CONSTRUCTOR(x300_radio_ctrl)
     , _ignore_cal_file(false)
 {
-    UHD_RFNOC_BLOCK_TRACE() << "x300_radio_ctrl_impl::ctor() " << std::endl;
+    UHD_RFNOC_BLOCK_TRACE() << "x300_radio_ctrl_impl::ctor() " ;
 
     ////////////////////////////////////////////////////////////////////
     // Set up basic info
@@ -75,20 +65,53 @@ UHD_RFNOC_RADIO_BLOCK_CONSTRUCTOR(x300_radio_ctrl)
     // Setup peripherals
     ////////////////////////////////////////////////////////////////
     _spi = spi_core_3000::make(ctrl,
-        radio_ctrl_impl::regs::sr_addr(radio_ctrl_impl::regs::SPI),
-        radio_ctrl_impl::regs::RB_SPI);
+        regs::sr_addr(radio_ctrl_impl::regs::SPI),
+        regs::rb_addr(radio_ctrl_impl::regs::RB_SPI)
+    );
     _adc = x300_adc_ctrl::make(_spi, DB_ADC_SEN);
     _dac = x300_dac_ctrl::make(_spi, DB_DAC_SEN, _radio_clk_rate);
 
     if (_radio_type==PRIMARY) {
-        _fp_gpio = gpio_atr::gpio_atr_3000::make(ctrl, regs::sr_addr(regs::FP_GPIO), regs::RB_FP_GPIO);
-        BOOST_FOREACH(const gpio_atr::gpio_attr_map_t::value_type attr, gpio_atr::gpio_attr_map) {
-            _tree->create<uint32_t>(fs_path("gpio") / "FP0" / attr.second)
-                .set(0)
-                .add_coerced_subscriber(boost::bind(&gpio_atr::gpio_atr_3000::set_gpio_attr, _fp_gpio, attr.first, _1));
+        _fp_gpio = gpio_atr::gpio_atr_3000::make(ctrl,
+            regs::sr_addr(regs::FP_GPIO),
+            regs::rb_addr(regs::RB_FP_GPIO)
+        );
+        for(const gpio_atr::gpio_attr_map_t::value_type attr:  gpio_atr::gpio_attr_map) {
+            switch (attr.first){
+                case usrp::gpio_atr::GPIO_SRC:
+                    _tree->create<std::vector<std::string>>(fs_path("gpio") / "FP0" / attr.second)
+                        .set(std::vector<std::string>(32, usrp::gpio_atr::default_attr_value_map.at(attr.first)))
+                        .add_coerced_subscriber([this](const std::vector<std::string>&){
+                             throw uhd::runtime_error("This device does not support setting the GPIO_SRC attribute.");
+                    });
+                    break;
+                case usrp::gpio_atr::GPIO_CTRL:
+                case usrp::gpio_atr::GPIO_DDR:
+                    _tree->create<std::vector<std::string>>(fs_path("gpio") / "FP0" / attr.second)
+                         .set(std::vector<std::string>(32, usrp::gpio_atr::default_attr_value_map.at(attr.first)))
+                         .add_coerced_subscriber([this, attr](const std::vector<std::string> str_val){
+                             uint32_t val = 0;
+                             for(size_t i = 0 ; i < str_val.size() ; i++){
+                                val += usrp::gpio_atr::gpio_attr_value_pair.at(attr.second).at(str_val[i])<<i;
+                            }
+                             _fp_gpio->set_gpio_attr(attr.first, val);
+                         });
+                    break;
+                case usrp::gpio_atr::GPIO_READBACK:
+                    _tree->create<uint32_t>(fs_path("gpio") / "FP0" / "READBACK")
+                        .set_publisher([this](){
+                            return _fp_gpio->read_gpio();
+                        });
+                    break;
+                default:
+                    _tree->create<uint32_t>(fs_path("gpio") / "FP0" / attr.second)
+                         .set(0)
+                         .add_coerced_subscriber([this, attr](const uint32_t val){
+                             _fp_gpio->set_gpio_attr(attr.first, val);
+                         });
+            }
+
         }
-        _tree->create<uint32_t>(fs_path("gpio") / "FP0" / "READBACK")
-            .set_publisher(boost::bind(&gpio_atr::gpio_atr_3000::read_gpio, _fp_gpio));
     }
 
     ////////////////////////////////////////////////////////////////
@@ -149,10 +172,9 @@ x300_radio_ctrl_impl::~x300_radio_ctrl_impl()
     _tree->remove(_root_path / "rx_fe_corrections");
     _tree->remove(_root_path / "tx_fe_corrections");
     if (_radio_type==PRIMARY) {
-        BOOST_FOREACH(const gpio_atr::gpio_attr_map_t::value_type attr, gpio_atr::gpio_attr_map) {
+        for(const gpio_atr::gpio_attr_map_t::value_type attr:  gpio_atr::gpio_attr_map) {
             _tree->remove(fs_path("gpio") / "FP0" / attr.second);
         }
-        _tree->remove(fs_path("gpio") / "FP0" / "READBACK");
     }
 
     // Reset peripherals
@@ -171,7 +193,7 @@ double x300_radio_ctrl_impl::set_rate(double rate)
 {
     const double actual_rate = get_rate();
     if (not uhd::math::frequencies_are_equal(rate, actual_rate)) {
-        UHD_MSG(warning) << "[X300 Radio] Requesting invalid sampling rate from device: " << rate/1e6 << " MHz. Actual rate is: " << actual_rate/1e6 << " MHz." << std::endl;
+        UHD_LOGGER_WARNING("X300 RADIO") << "Requesting invalid sampling rate from device: " << rate/1e6 << " MHz. Actual rate is: " << actual_rate/1e6 << " MHz." ;
     }
     // On X3x0, tick rate can't actually be changed at runtime
     return actual_rate;
@@ -228,6 +250,20 @@ double x300_radio_ctrl_impl::get_rx_frequency(const size_t chan)
     ).get();
 }
 
+double x300_radio_ctrl_impl::set_rx_bandwidth(const double bandwidth, const size_t chan)
+{
+    return _tree->access<double>(
+        fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name / "bandwidth" / "value")
+    ).set(bandwidth).get();
+}
+
+double x300_radio_ctrl_impl::get_rx_bandwidth(const size_t chan)
+{
+    return _tree->access<double>(
+        fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name / "bandwidth" / "value")
+    ).get();
+}
+
 double x300_radio_ctrl_impl::set_tx_gain(const double gain, const size_t chan)
 {
     //TODO: This is extremely hacky!
@@ -238,7 +274,7 @@ double x300_radio_ctrl_impl::set_tx_gain(const double gain, const size_t chan)
         radio_ctrl_impl::set_tx_gain(actual_gain, chan);
         return gain;
     } else {
-        UHD_MSG(warning) << "set_tx_gain: could not apply gain for this daughterboard.";
+        UHD_LOGGER_WARNING("X300 RADIO") << "set_tx_gain: could not apply gain for this daughterboard.";
         radio_ctrl_impl::set_tx_gain(0.0, chan);
         return 0.0;
     }
@@ -254,19 +290,209 @@ double x300_radio_ctrl_impl::set_rx_gain(const double gain, const size_t chan)
         radio_ctrl_impl::set_rx_gain(actual_gain, chan);
         return gain;
     } else {
-        UHD_MSG(warning) << "set_rx_gain: could not apply gain for this daughterboard.";
+        UHD_LOGGER_WARNING("X300 RADIO") << "set_rx_gain: could not apply gain for this daughterboard.";
         radio_ctrl_impl::set_tx_gain(0.0, chan);
         return 0.0;
     }
 }
 
 
+std::vector<std::string> x300_radio_ctrl_impl::get_rx_lo_names(const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    std::vector<std::string> lo_names;
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        for(const std::string &name:  _tree->list(rx_fe_fe_root / "los")) {
+            lo_names.push_back(name);
+        }
+    }
+    return lo_names;
+}
+
+std::vector<std::string> x300_radio_ctrl_impl::get_rx_lo_sources(const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            if (_tree->exists(rx_fe_fe_root / "los" / ALL_LOS)) {
+                //Special value ALL_LOS support atomically sets the source for all LOs
+                return _tree->access< std::vector<std::string> >(rx_fe_fe_root / "los" / ALL_LOS / "source" / "options").get();
+            } else {
+                return std::vector<std::string>();
+            }
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                return _tree->access< std::vector<std::string> >(rx_fe_fe_root / "los" / name / "source" / "options").get();
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        // If the daughterboard doesn't expose it's LO(s) then it can only be internal
+        return std::vector<std::string> (1, "internal");
+    }
+}
+
+void x300_radio_ctrl_impl::set_rx_lo_source(const std::string &src, const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            if (_tree->exists(rx_fe_fe_root / "los" / ALL_LOS)) {
+                //Special value ALL_LOS support atomically sets the source for all LOs
+                _tree->access<std::string>(rx_fe_fe_root / "los" / ALL_LOS / "source" / "value").set(src);
+            } else {
+                for(const std::string &n:  _tree->list(rx_fe_fe_root / "los")) {
+                    this->set_rx_lo_source(src, n, chan);
+                }
+            }
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                _tree->access<std::string>(rx_fe_fe_root / "los" / name / "source" / "value").set(src);
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        throw uhd::runtime_error("This device does not support manual configuration of LOs");
+    }
+}
+
+const std::string x300_radio_ctrl_impl::get_rx_lo_source(const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            //Special value ALL_LOS support atomically sets the source for all LOs
+            return _tree->access<std::string>(rx_fe_fe_root / "los" / ALL_LOS / "source" / "value").get();
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                return _tree->access<std::string>(rx_fe_fe_root / "los" / name / "source" / "value").get();
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        // If the daughterboard doesn't expose it's LO(s) then it can only be internal
+        return "internal";
+    }
+}
+
+void x300_radio_ctrl_impl::set_rx_lo_export_enabled(bool enabled, const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            if (_tree->exists(rx_fe_fe_root / "los" / ALL_LOS)) {
+                //Special value ALL_LOS support atomically sets the source for all LOs
+                _tree->access<bool>(rx_fe_fe_root / "los" / ALL_LOS / "export").set(enabled);
+            } else {
+                for(const std::string &n:  _tree->list(rx_fe_fe_root / "los")) {
+                    this->set_rx_lo_export_enabled(enabled, n, chan);
+                }
+            }
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                _tree->access<bool>(rx_fe_fe_root / "los" / name / "export").set(enabled);
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        throw uhd::runtime_error("This device does not support manual configuration of LOs");
+    }
+}
+
+bool x300_radio_ctrl_impl::get_rx_lo_export_enabled(const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            //Special value ALL_LOS support atomically sets the source for all LOs
+            return _tree->access<bool>(rx_fe_fe_root / "los" / ALL_LOS / "export").get();
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                return _tree->access<bool>(rx_fe_fe_root / "los" / name / "export").get();
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        // If the daughterboard doesn't expose it's LO(s), assume it cannot export
+        return false;
+    }
+}
+
+double x300_radio_ctrl_impl::set_rx_lo_freq(double freq, const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            throw uhd::runtime_error("LO frequency must be set for each stage individually");
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                _tree->access<double>(rx_fe_fe_root / "los" / name / "freq" / "value").set(freq);
+                return _tree->access<double>(rx_fe_fe_root / "los" / name / "freq" / "value").get();
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        throw uhd::runtime_error("This device does not support manual configuration of LOs");
+    }
+}
+
+double x300_radio_ctrl_impl::get_rx_lo_freq(const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            throw uhd::runtime_error("LO frequency must be retrieved for each stage individually");
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                return _tree->access<double>(rx_fe_fe_root / "los" / name / "freq" / "value").get();
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        // Return actual RF frequency if the daughterboard doesn't expose it's LO(s)
+        return _tree->access<double>(rx_fe_fe_root / "freq" /" value").get();
+    }
+}
+
+freq_range_t x300_radio_ctrl_impl::get_rx_lo_freq_range(const std::string &name, const size_t chan)
+{
+    fs_path rx_fe_fe_root = fs_path("dboards" / _radio_slot / "rx_frontends" / _rx_fe_map.at(chan).db_fe_name);
+
+    if (_tree->exists(rx_fe_fe_root / "los")) {
+        if (name == ALL_LOS) {
+            throw uhd::runtime_error("LO frequency range must be retrieved for each stage individually");
+        } else {
+            if (_tree->exists(rx_fe_fe_root / "los")) {
+                return _tree->access<freq_range_t>(rx_fe_fe_root / "los" / name / "freq" / "range").get();
+            } else {
+                throw uhd::runtime_error("Could not find LO stage " + name);
+            }
+        }
+    } else {
+        // Return the actual RF range if the daughterboard doesn't expose it's LO(s)
+        return _tree->access<meta_range_t>(rx_fe_fe_root / "freq" / "range").get();
+    }
+}
+
 template <typename map_type>
 static size_t _get_chan_from_map(std::map<size_t, map_type> map, const std::string &fe)
 {
-    // TODO replace with 'auto' when possible
-    typedef typename std::map<size_t, map_type>::iterator chan_iterator;
-    for (chan_iterator it = map.begin(); it != map.end(); ++it) {
+    for (auto it = map.begin(); it != map.end(); ++it) {
         if (it->second.db_fe_name == fe) {
             return it->first;
         }
@@ -318,6 +544,68 @@ double x300_radio_ctrl_impl::get_output_samp_rate(size_t chan)
     return _rx_fe_map.at(chan).core->get_output_rate();
 }
 
+std::vector<std::string> x300_radio_ctrl_impl::get_gpio_banks() const
+{
+    std::vector<std::string> banks{"RX", "TX"};
+    // These pairs are the same, but RXA/TXA are from pre-rfnoc era and are kept for backward compat:
+    banks.push_back("RX"+_radio_slot);
+    banks.push_back("TX"+_radio_slot);
+    if (_fp_gpio) {
+        banks.push_back("FP0");
+    }
+    return banks;
+}
+
+void x300_radio_ctrl_impl::set_gpio_attr(
+        const std::string &bank,
+        const std::string &attr,
+        const uint32_t value,
+        const uint32_t mask
+) {
+    if (bank == "FP0" and _fp_gpio) {
+        const uint32_t current = _tree->access<uint32_t>(fs_path("gpio") / bank / attr).get();
+        const uint32_t new_value = (current & ~mask) | (value & mask);
+        _tree->access<uint32_t>(fs_path("gpio") / bank / attr).set(new_value);
+        return;
+    }
+    if (bank.size() > 2 and bank[1] == 'X')
+    {
+        const std::string name = bank.substr(2);
+        const dboard_iface::unit_t unit = (bank[0] == 'R')? dboard_iface::UNIT_RX : dboard_iface::UNIT_TX;
+        dboard_iface::sptr iface = _tree->access<dboard_iface::sptr>(fs_path("dboards") / name / "iface").get();
+        if (attr == "CTRL") iface->set_pin_ctrl(unit, uint16_t(value), uint16_t(mask));
+        if (attr == "DDR") iface->set_gpio_ddr(unit, uint16_t(value), uint16_t(mask));
+        if (attr == "OUT") iface->set_gpio_out(unit, uint16_t(value), uint16_t(mask));
+        if (attr == "ATR_0X") iface->set_atr_reg(unit, gpio_atr::ATR_REG_IDLE, uint16_t(value), uint16_t(mask));
+        if (attr == "ATR_RX") iface->set_atr_reg(unit, gpio_atr::ATR_REG_RX_ONLY, uint16_t(value), uint16_t(mask));
+        if (attr == "ATR_TX") iface->set_atr_reg(unit, gpio_atr::ATR_REG_TX_ONLY, uint16_t(value), uint16_t(mask));
+        if (attr == "ATR_XX") iface->set_atr_reg(unit, gpio_atr::ATR_REG_FULL_DUPLEX, uint16_t(value), uint16_t(mask));
+    }
+}
+
+uint32_t x300_radio_ctrl_impl::get_gpio_attr(
+        const std::string &bank,
+        const std::string &attr
+) {
+    if (bank == "FP0" and _fp_gpio) {
+        return uint32_t(_tree->access<uint64_t>(fs_path("gpio") / bank / attr).get());
+    }
+    if (bank.size() > 2 and bank[1] == 'X') {
+        const std::string name = bank.substr(2);
+        const dboard_iface::unit_t unit = (bank[0] == 'R')? dboard_iface::UNIT_RX : dboard_iface::UNIT_TX;
+        dboard_iface::sptr iface = _tree->access<dboard_iface::sptr>(fs_path("dboards") / name / "iface").get();
+        if (attr == "CTRL") return iface->get_pin_ctrl(unit);
+        if (attr == "DDR") return iface->get_gpio_ddr(unit);
+        if (attr == "OUT") return iface->get_gpio_out(unit);
+        if (attr == "ATR_0X") return iface->get_atr_reg(unit, gpio_atr::ATR_REG_IDLE);
+        if (attr == "ATR_RX") return iface->get_atr_reg(unit, gpio_atr::ATR_REG_RX_ONLY);
+        if (attr == "ATR_TX") return iface->get_atr_reg(unit, gpio_atr::ATR_REG_TX_ONLY);
+        if (attr == "ATR_XX") return iface->get_atr_reg(unit, gpio_atr::ATR_REG_FULL_DUPLEX);
+        if (attr == "READBACK") return iface->read_gpio(unit);
+    }
+    return 0;
+}
+
 /****************************************************************************
  * Radio control and setup
  ***************************************************************************/
@@ -337,10 +625,16 @@ void x300_radio_ctrl_impl::setup_radio(
     static const size_t RX_EEPROM_ADDR  = 0x5;
     static const size_t TX_EEPROM_ADDR  = 0x4;
     static const size_t GDB_EEPROM_ADDR = 0x1;
-    const static std::vector<size_t> EEPROM_ADDRS =
-        boost::assign::list_of(RX_EEPROM_ADDR)(TX_EEPROM_ADDR)(GDB_EEPROM_ADDR);
-    const static std::vector<std::string> EEPROM_PATHS =
-        boost::assign::list_of("rx_eeprom")("tx_eeprom")("gdb_eeprom");
+    const static std::vector<size_t> EEPROM_ADDRS{
+        RX_EEPROM_ADDR,
+        TX_EEPROM_ADDR,
+        GDB_EEPROM_ADDR
+    };
+    const static std::vector<std::string> EEPROM_PATHS{
+        "rx_eeprom",
+        "tx_eeprom",
+        "gdb_eeprom"
+    };
 
     const size_t DB_OFFSET = (_radio_slot == "A") ? 0x0 : 0x2;
     const fs_path db_path = ("dboards" / _radio_slot);
@@ -358,7 +652,9 @@ void x300_radio_ctrl_impl::setup_radio(
     //create a new dboard interface
     x300_dboard_iface_config_t db_config;
     db_config.gpio = gpio_atr::db_gpio_atr_3000::make(_get_ctrl(IO_MASTER_RADIO),
-        radio_ctrl_impl::regs::sr_addr(radio_ctrl_impl::regs::GPIO), radio_ctrl_impl::regs::RB_DB_GPIO);
+        radio_ctrl_impl::regs::sr_addr(radio_ctrl_impl::regs::GPIO),
+        radio_ctrl_impl::regs::rb_addr(radio_ctrl_impl::regs::RB_DB_GPIO)
+    );
     db_config.spi = _spi;
     db_config.rx_spi_slaveno = DB_RX_SEN;
     db_config.tx_spi_slaveno = DB_TX_SEN;
@@ -380,7 +676,7 @@ void x300_radio_ctrl_impl::setup_radio(
     );
 
     size_t rx_chan = 0, tx_chan = 0;
-    BOOST_FOREACH(const std::string& fe, _db_manager->get_rx_frontends()) {
+    for(const std::string& fe:  _db_manager->get_rx_frontends()) {
         if (rx_chan >= _get_num_radios()) {
             break;
         }
@@ -393,7 +689,7 @@ void x300_radio_ctrl_impl::setup_radio(
         _rx_fe_map[rx_chan].core->set_fe_connection(usrp::fe_connection_t(conn, if_freq));
         rx_chan++;
     }
-    BOOST_FOREACH(const std::string& fe, _db_manager->get_tx_frontends()) {
+    for(const std::string& fe:  _db_manager->get_tx_frontends()) {
         if (tx_chan >= _get_num_radios()) {
             break;
         }
@@ -515,10 +811,10 @@ void x300_radio_ctrl_impl::self_test_adc(uint32_t ramp_time_ms)
     _adc->set_test_word("ramp", "ramp");
     _regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 0);
     //Sleep added for SPI transactions to finish and ramp to start before checker is enabled.
-    boost::this_thread::sleep(boost::posix_time::microsec(1000));
+    std::this_thread::sleep_for(std::chrono::microseconds(1000));
     _regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 1);
 
-    boost::this_thread::sleep(boost::posix_time::milliseconds(ramp_time_ms));
+    std::this_thread::sleep_for(std::chrono::milliseconds(ramp_time_ms));
     _regs->misc_ins_reg.refresh();
 
     std::string i_status, q_status;
@@ -555,31 +851,26 @@ void x300_radio_ctrl_impl::self_test_adc(uint32_t ramp_time_ms)
 void x300_radio_ctrl_impl::extended_adc_test(const std::vector<x300_radio_ctrl_impl::sptr>& radios, double duration_s)
 {
     static const size_t SECS_PER_ITER = 5;
-    UHD_MSG(status) << boost::format("Running Extended ADC Self-Test (Duration=%.0fs, %ds/iteration)...\n")
+    UHD_LOGGER_INFO("X300 RADIO") << boost::format("Running Extended ADC Self-Test (Duration=%.0fs, %ds/iteration)...")
         % duration_s % SECS_PER_ITER;
 
     size_t num_iters = static_cast<size_t>(ceil(duration_s/SECS_PER_ITER));
     size_t num_failures = 0;
     for (size_t iter = 0; iter < num_iters; iter++) {
-        //Print date and time
-        boost::posix_time::time_facet *facet = new boost::posix_time::time_facet("%d-%b-%Y %H:%M:%S");
-        std::ostringstream time_strm;
-        time_strm.imbue(std::locale(std::locale::classic(), facet));
-        time_strm << boost::posix_time::second_clock::local_time();
         //Run self-test
-        UHD_MSG(status) << boost::format("-- [%s] Iteration %06d... ") % time_strm.str() % (iter+1);
+        UHD_LOGGER_INFO("X300 RADIO") << boost::format("Extended ADC Self-Test Iteration %06d... ") % (iter+1);
         try {
             for (size_t i = 0; i < radios.size(); i++) {
                 radios[i]->self_test_adc((SECS_PER_ITER*1000)/radios.size());
             }
-            UHD_MSG(status) << "passed" << std::endl;
+            UHD_LOGGER_INFO("X300 RADIO") << boost::format("Extended ADC Self-Test Iteration %06d passed ") % (iter+1);
         } catch(std::exception &e) {
             num_failures++;
-            UHD_MSG(status) << e.what() << std::endl;
+            UHD_LOGGER_ERROR("X300 RADIO") << e.what();
         }
     }
     if (num_failures == 0) {
-        UHD_MSG(status) << "Extended ADC Self-Test PASSED\n";
+        UHD_LOGGER_INFO("X300 RADIO") << "Extended ADC Self-Test PASSED";
     } else {
         throw uhd::runtime_error(
                 (boost::format("Extended ADC Self-Test FAILED!!! (%d/%d failures)\n") % num_failures % num_iters).str());
@@ -642,7 +933,7 @@ void x300_radio_ctrl_impl::synchronize_dacs(const std::vector<x300_radio_ctrl_im
             }
 
             //Wait and check status
-            boost::this_thread::sleep(boost::posix_time::microseconds(t_sync_us));
+            std::this_thread::sleep_for(std::chrono::microseconds(t_sync_us));
             for (size_t i = 0; i < radios.size(); i++) {
                 radios[i]->_dac->verify_sync();
             }
@@ -652,7 +943,7 @@ void x300_radio_ctrl_impl::synchronize_dacs(const std::vector<x300_radio_ctrl_im
         catch (const uhd::runtime_error &e)
         {
             err_str = e.what();
-            UHD_LOGV(rarely) << "X300 RADIO: Retrying DAC synchronization: " << err_str;
+            UHD_LOGGER_TRACE("X300 RADIO") << "Retrying DAC synchronization: " << err_str;
         }
     }
     throw uhd::runtime_error(err_str);
@@ -664,7 +955,7 @@ double x300_radio_ctrl_impl::self_cal_adc_xfer_delay(
     boost::function<void(double)> wait_for_clk_locked,
     bool apply_delay)
 {
-    UHD_MSG(status) << "Running ADC transfer delay self-cal: " << std::flush;
+    UHD_LOGGER_INFO("X300 RADIO") << "Running ADC transfer delay self-cal: ";
 
     //Effective resolution of the self-cal.
     static const size_t NUM_DELAY_STEPS = 100;
@@ -674,7 +965,6 @@ double x300_radio_ctrl_impl::self_cal_adc_xfer_delay(
     double delay_range = 2 * master_clk_period;
     double delay_incr = delay_range / NUM_DELAY_STEPS;
 
-    UHD_MSG(status) << "Measuring..." << std::flush;
     double cached_clk_delay = clock->get_clock_delay(X300_CLOCK_WHICH_ADC0);
     double fpga_clk_delay = clock->get_clock_delay(X300_CLOCK_WHICH_FPGA);
 
@@ -698,7 +988,7 @@ double x300_radio_ctrl_impl::self_cal_adc_xfer_delay(
             radios[r]->_regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 0);
             radios[r]->_regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 1);
             //50ms @ 200MHz = 10 million samples
-            boost::this_thread::sleep(boost::posix_time::milliseconds(50));
+            std::this_thread::sleep_for(std::chrono::milliseconds(50));
             if (radios[r]->_regs->misc_ins_reg.read(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_I_LOCKED)) {
                 err_code += radios[r]->_regs->misc_ins_reg.get(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_I_ERROR);
             } else {
@@ -713,14 +1003,14 @@ double x300_radio_ctrl_impl::self_cal_adc_xfer_delay(
             radios[r]->_regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 0);
             radios[r]->_regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 1);
             //50ms @ 200MHz = 10 million samples
-            boost::this_thread::sleep(boost::posix_time::milliseconds(50));
+            std::this_thread::sleep_for(std::chrono::milliseconds(50));
             if (radios[r]->_regs->misc_ins_reg.read(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_Q_LOCKED)) {
                 err_code += radios[r]->_regs->misc_ins_reg.get(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_Q_ERROR);
             } else {
                 err_code += 100;    //Increment error code by 100 to indicate no lock
             }
         }
-        //UHD_MSG(status) << (boost::format("XferDelay=%fns, Error=%d\n") % delay % err_code);
+        //UHD_LOGGER_INFO("X300 RADIO") << (boost::format("XferDelay=%fns, Error=%d") % delay % err_code);
         results.push_back(std::pair<double,bool>(delay, err_code==0));
     }
 
@@ -772,7 +1062,6 @@ double x300_radio_ctrl_impl::self_cal_adc_xfer_delay(
     }
 
     if (apply_delay) {
-        UHD_MSG(status) << "Validating..." << std::flush;
         //Apply delay
         win_center = clock->set_clock_delay(X300_CLOCK_WHICH_ADC0, win_center);  //Sets ADC0 and ADC1
         wait_for_clk_locked(0.1);
@@ -790,7 +1079,7 @@ double x300_radio_ctrl_impl::self_cal_adc_xfer_delay(
         radios[r]->_adc->set_test_word("normal", "normal");
         radios[r]->_regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 0);
     }
-    UHD_MSG(status) << (boost::format(" done (FPGA->ADC=%.3fns%s, Window=%.3fns)\n") %
+    UHD_LOGGER_INFO("X300 RADIO") << (boost::format("ADC transfer delay self-cal done (FPGA->ADC=%.3fns%s, Window=%.3fns)") %
         (win_center-fpga_clk_delay) % (cycle_slip?" +cyc":"") % win_length);
 
     return win_center;
@@ -814,7 +1103,7 @@ void x300_radio_ctrl_impl::_update_atr_leds(const std::string &rx_ant, const siz
 
 void x300_radio_ctrl_impl::_self_cal_adc_capture_delay(bool print_status)
 {
-    if (print_status) UHD_MSG(status) << "Running ADC capture delay self-cal..." << std::flush;
+    if (print_status) UHD_LOGGER_INFO("X300 RADIO") << "Running ADC capture delay self-cal...";
 
     static const uint32_t NUM_DELAY_STEPS = 32;   //The IDELAYE2 element has 32 steps
     static const uint32_t NUM_RETRIES     = 2;    //Retry self-cal if it fails in warmup situations
@@ -838,8 +1127,8 @@ void x300_radio_ctrl_impl::_self_cal_adc_capture_delay(bool print_status)
             //and count deviations from the expected value
             _regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 0);
             _regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 1);
-            //10ms @ 200MHz = 2 million samples
-            boost::this_thread::sleep(boost::posix_time::milliseconds(10));
+            //5ms @ 200MHz = 1 million samples
+            std::this_thread::sleep_for(std::chrono::milliseconds(5));
             if (_regs->misc_ins_reg.read(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER0_I_LOCKED)) {
                 err_code += _regs->misc_ins_reg.get(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER0_I_ERROR);
             } else {
@@ -853,8 +1142,8 @@ void x300_radio_ctrl_impl::_self_cal_adc_capture_delay(bool print_status)
             //and count deviations from the expected value
             _regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 0);
             _regs->misc_outs_reg.write(radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, 1);
-            //10ms @ 200MHz = 2 million samples
-            boost::this_thread::sleep(boost::posix_time::milliseconds(10));
+            //5ms @ 200MHz = 1 million samples
+            std::this_thread::sleep_for(std::chrono::milliseconds(5));
             if (_regs->misc_ins_reg.read(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER0_Q_LOCKED)) {
                 err_code += _regs->misc_ins_reg.get(radio_regmap_t::misc_ins_reg_t::ADC_CHECKER0_Q_ERROR);
             } else {
@@ -877,14 +1166,14 @@ void x300_radio_ctrl_impl::_self_cal_adc_capture_delay(bool print_status)
                     }
                 }
             }
-            //UHD_MSG(status) << (boost::format("CapTap=%d, Error=%d\n") % dly_tap % err_code);
+            //UHD_LOGGER_INFO("X300 RADIO") << (boost::format("CapTap=%d, Error=%d") % dly_tap % err_code);
         }
 
         //Retry the self-cal if it fails
         if ((win_start == -1 || (win_stop - win_start) < MIN_WINDOW_LEN) && iter < NUM_RETRIES /*not last iteration*/) {
             win_start = -1;
             win_stop = -1;
-            boost::this_thread::sleep(boost::posix_time::milliseconds(2000));
+            std::this_thread::sleep_for(std::chrono::milliseconds(2000));
         } else {
             break;
         }
@@ -907,7 +1196,7 @@ void x300_radio_ctrl_impl::_self_cal_adc_capture_delay(bool print_status)
 
     if (print_status) {
         double tap_delay = (1.0e12 / _radio_clk_rate) / (2*32); //in ps
-        UHD_MSG(status) << boost::format(" done (Tap=%d, Window=%d, TapDelay=%.3fps, Iter=%d)\n") % ideal_tap % (win_stop-win_start) % tap_delay % iter;
+        UHD_LOGGER_INFO("X300 RADIO") << boost::format("ADC capture delay self-cal done (Tap=%d, Window=%d, TapDelay=%.3fps, Iter=%d)") % ideal_tap % (win_stop-win_start) % tap_delay % iter;
     }
 }
 
@@ -916,7 +1205,7 @@ void x300_radio_ctrl_impl::_check_adc(const uint32_t val)
     //Wait for previous control transaction to flush
     user_reg_read64(regs::RB_TEST);
     //Wait for ADC test pattern to propagate
-    boost::this_thread::sleep(boost::posix_time::microsec(5));
+    std::this_thread::sleep_for(std::chrono::microseconds(5));
     //Read value of RX readback register and verify
     uint32_t adc_rb = static_cast<uint32_t>(user_reg_read64(regs::RB_TEST)>>32);
     adc_rb ^= 0xfffc0000; //adapt for I inversion in FPGA
@@ -941,9 +1230,9 @@ void x300_radio_ctrl_impl::_set_command_time(const time_spec_t &spec, const size
  ***************************************************************************/
 bool x300_radio_ctrl_impl::check_radio_config()
 {
-    UHD_RFNOC_BLOCK_TRACE() << "x300_radio_ctrl_impl::check_radio_config() " << std::endl;
+    UHD_RFNOC_BLOCK_TRACE() << "x300_radio_ctrl_impl::check_radio_config() " ;
     const fs_path rx_fe_path = fs_path("dboards" / _radio_slot / "rx_frontends");
-    for (size_t chan = 0; chan < _get_num_radios(); chan++) {
+    for (size_t chan = 0; chan < _num_rx_channels; chan++) {
         if (_tree->exists(rx_fe_path / _rx_fe_map.at(chan).db_fe_name / "enabled")) {
             const bool chan_active = _is_streamer_active(uhd::RX_DIRECTION, chan);
             if (chan_active) {
@@ -955,7 +1244,7 @@ bool x300_radio_ctrl_impl::check_radio_config()
     }
 
     const fs_path tx_fe_path = fs_path("dboards" / _radio_slot / "tx_frontends");
-    for (size_t chan = 0; chan < _get_num_radios(); chan++) {
+    for (size_t chan = 0; chan < _num_tx_channels; chan++) {
         if (_tree->exists(tx_fe_path / _tx_fe_map.at(chan).db_fe_name / "enabled")) {
             const bool chan_active = _is_streamer_active(uhd::TX_DIRECTION, chan);
             if (chan_active) {
