@@ -1,19 +1,9 @@
 #!/usr/bin/env python
 #
 # Copyright 2015-2016 Ettus Research LLC
+# Copyright 2018 Ettus Research, a National Instruments Company
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 #
 """ Run uhd_find_devices and parse the output. """
 
@@ -36,6 +26,8 @@ def get_usrp_list(device_filter=None, env=None):
         if not re.match("Device Address", uhd_string):
             continue
         this_result = {k: v for k, v in re.findall("    ([a-z]+): (.*)", uhd_string)}
+        if this_result.get('reachable') == "No":
+            continue
         args_string = ""
         try:
             args_string = "type={},serial={}".format(this_result['type'], this_result['serial'])
